#####################################################
### ANALYSE PLANT-TO-PLANT-VARIABILITY            ####
### Author: Diana Trutschel & Susann Moenchgesang #####
#######################################################


###need libraries
library(lme4)#DT
library(ggplot2)#for plotting
#function to convert factor into numeric
as.numeric.factor <- function(x) {as.numeric(levels(x))[x]}

##################################################################
#####   Variances for LC/MS targeted data         #########################
#################################################################

## assume the same variance for each accession

#read in table with experimental design and log-transformed values
#the data file has the dimension of nr x nc, 
#nr = number of measurements
#nc  = number of features (subtances) + number of analysed variance levels
#column 1 - accession Bur-0 - Zu-0
#column 2 - experimental batch 1-3
#column 3 - plant number 1-4
#column 4 - x log-transformed metabolite abundance

#complete data, with no Na (0)
data<-read.csv("rootsTarget.csv",row.names = 1)
dim(data)

#read in table with substance defined substance classes corresponds to each measured feature in data file
#the data file has the dimension of 1 x nc, 
#nc  = number of features (subtances) 
data.Substanz<-read.csv("GSLFlavPP.csv",row.names = 1)

#convert to factors for using lmm
data$plant<-as.factor(data$plant) 
data$eco<-as.factor(data$eco)
data$batch<-as.factor(data$batch)

#estimate variances in each level for each and for each feature i
res<-sapply(1:(dim(data)[2]-3), function(i){
  
  val<-data[,c(1:3,3+i)]
  colnames(val)[4]<-"values"
  
  lm1<-lmer(values ~ 1+(1|eco/batch), data=val)#nested design
  var<-c(as.data.frame(summary(lm1)$varcor)[,"vcov"], 
         sum(as.data.frame(summary(lm1)$varcor)[,"vcov"]))
  names(var)<-c("batch", "eco","plant","total")
  return(var)})
colnames(res)<-colnames(data)[4:dim(data)[2]]

#save variance in data file, where 
#number of rows = number of variance levels + 1 (for total variance)
#number of columns = number of analysed features
write.csv(res, file="variancesGSLFlav")

#mean variances of each level is needed for plot
var.means<-apply(res, 1, function(x){round(mean(x),3)}) # batch, eco, plant, total

#transform variance estimate data into a data frame for plotting, where
#number of rows = number of features x number of analysed variance levels
#number of columns = 3 (variance estimates, level of variance, analysed feature)
var.plot<-data.frame(var=as.vector(res),
                     level=rep(rownames(res), times=dim(res)[2]),
                     signal=rep(1:dim(res)[2], each=dim(res)[1])
)
var.plot$level<-factor(var.plot$level, levels=c("plant","batch", "eco",  "total")) #correct order

#combine variance estimates for each feature with information of substance class for each feature
var.plot.subst<-var.plot
var.plot.subst<-cbind(var.plot.subst,Substance=rep(unlist(data.Substanz),each=4))

###strip chart for each variance level
pd <- position_dodge(0.4)
ggplot(var.plot.subst, 
       aes(x= level,y=var, 
           group=Substance, col=level,shape=Substance))+
  geom_point(position=pd) +
  xlab("") + 
  ylab("estimated variance from lmm")+
  annotate("text", x=c(2,3,1,4), y=20, 
           label= var.means , col=c("red", "green", "blue", "violet")) +
  annotate("segment", x=c(1.6,2.6,0.6,3.6), xend=c(2.4,3.4,1.4,4.4), y=var.means, yend=var.means,
           col=c("red", "green", "blue", "violet")) +
  scale_colour_manual(values=c("blue", "red","green", "violet"))+
  theme_bw()+
  theme(panel.grid.minor.x=element_blank(), 
        panel.grid.major.x=element_blank(),
        plot.title = element_text( lineheight=1,face="bold", size=16),
        axis.text.x = element_text(size=12),
        axis.text.y = element_text(size=12),
        axis.title.y = element_text(size=16),
        legend.title = element_text(size=12),
        legend.position="top"
  )

###cumulative (plant) ICC distribution plot
ICC<-res["plant",]/res["total",]
ICC_sort<-sort(ICC)

pers60<-round(ICC_sort[length(ICC)*0.25],2)
pers80<-round(ICC_sort[length(ICC)*0.5],2)
pers90<-round(ICC_sort[length(ICC)*0.75],2)
plot(x=(1:length(ICC_sort))/length(ICC_sort),y=ICC_sort, type="l",
     xlab="fraction of features", ylab="ICC",cex=0.2 , cex.main=1.9,cex.lab=1.5)
text(pos=2,y=c(pers60+0.04,pers80+0.04,pers90+0.04),x=c(0.25,0.5,0.75),
     labels=c(pers60,pers80,pers90), cex=0.9)
segments(x0=round(1-c(0.75,0.5,0.25),2), 
         x1=round(1-c(0.75,0.5,0.25),2), 
         y0=-0.3,
         y1=c(pers60,pers80,pers90), lty=2)
segments(x0=c(0,0,0), 
         x1=round(1-c(0.75,0.5,0.25),2), 
         y0=c(pers60,pers80,pers90), 
         y1=c(pers60,pers80,pers90), lty=2)


###ICC strip charts#########
res2<-rbind(res,ICC=ICC)
ICC.plot<-data.frame(ICC=ICC, signal=1:dim(res)[2])
#combine variance estimates for each feature with information of substance class for each feature
ICC.plot.subst<-data.frame(ICC.plot,Substance=as.factor(unlist(data.Substanz)))
#mean ICC for each substance
(mean.subst<-sapply(levels(ICC.plot.subst$Substance), function(i){  
  mean(unlist(subset(ICC.plot.subst, subset=Substance==i, select=c("ICC"))))})
)

pd <- position_dodge(0.3)
ggplot(data=ICC.plot.subst, aes(x=Substance, y=ICC,  color=Substance, group=rownames(ICC.plot.subst), shape=Substance) )+
  scale_y_continuous(limits = c(0, 1))+
  geom_point(position=pd)+
  annotate("segment", x=c(0.6,1.6,2.6), xend=c(1.4,2.4,3.4), 
           y=mean.subst, yend=mean.subst, col=c("blue", "red","green"))+
  scale_colour_manual(values=c("blue", "red","green"))+
  theme_bw()+
  theme(panel.grid.minor.x=element_blank(), 
        panel.grid.major.x=element_blank(),
        plot.title = element_text( lineheight=1,face="bold", size=16),
        axis.text.x = element_text(size=12),
        axis.text.y = element_text(size=12),
        axis.title.y = element_text(size=16),
        axis.title.x=element_blank(),
        legend.title = element_text(size=12),
        legend.position="none"
  )


##determine (plant) ICC for each accession separately###########
###################################################################
ICC.eco.plot.all<-NULL
for(j in 1:length(levels(data$eco))){
  data_Eco<-subset(data, subset=eco==levels(eco)[j])
  print(j)
  ##make subset for each accession
  res<-sapply(1:(dim(data_Eco)[2]-3), function(i){
    
    val<-data_Eco[,c(1:3,3+i)]
    colnames(val)[4]<-"values"
    (lm1<-lmer(values ~ 1+(1|batch), data=val))#nested design
    var<-c(as.data.frame(summary(lm1)$varcor)[,"vcov"]/sum(as.data.frame(summary(lm1)$varcor)[,"vcov"]))
    names(var)<-c("ICC batch","ICC plant")
    return(var)})
  
  colnames(res)<-colnames(data_Eco)[4:dim(data)[2]]
  
  ICC.eco.plot<-data.frame(var=as.vector(res),
                           level=as.factor(rep(rownames(res), times=dim(res)[2])),
                           signal=as.factor(rep(1:dim(res)[2], each=dim(res)[1])),
                           eco=rep(levels(data$eco)[j], times=prod(dim(res)))
  )
  ICC.eco.plot.all<-rbind(ICC.eco.plot.all,ICC.eco.plot)
}
#save(var.eco.plot.all,file="var.eco.plot.all-icc.Rdata")
ICC.eco.plot.all$level<-factor(ICC.eco.plot.all$level, levels=c("ICC plant","ICC batch")) #correct order
ICC.eco.plot.all.subst<-cbind(ICC.eco.plot.all,substance=rep(rep(unlist(data.Substanz),each=2),times=19))
ICC.eco.plot.all.subst<-transform(ICC.eco.plot.all.subst, gr=paste(level,substance))

ggplot(subset(ICC.eco.plot.all.subst, subset=level=="ICC plant"),
       aes(x= eco,y=var
           , col=eco
           , shape=substance
       ))+
  geom_point(position=pd) +
  facet_wrap(~substance,nrow=3)+
  xlab("") +
  ylab("ICC")+
  ggtitle(" ") +
  theme_bw()+
  theme(panel.grid.minor.x=element_blank(),
        panel.grid.major.x=element_blank(),
        plot.title = element_text( lineheight=1,face="bold", size=14),
        axis.text.x = element_text(size=12, angle=90, vjust = 0.5),
        axis.text.x=element_blank(),
        axis.text.y = element_text(size=12),
        axis.title.y = element_text(size=16),
        legend.title = element_text(size=12),
        legend.position="none"
  )

